/* SPDX-License-Identifier: GPL-2.0-or-later */
/*
 * A tool to access KSZ9897 switch via I2C
 *
 * Copyright (C) 2018-2020 TQ-Systems GmbH <oss@ew.tq-group.com>, D-82229 Seefeld, Germany.
 * Author: Max Merchel
 *
 * inspired by i2c tools
 */

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <time.h>
#include <sys/ioctl.h>
#include <linux/i2c.h>
#include <linux/i2c-dev.h>

#define DEFAULT_I2C_BUS		"/dev/i2c-0"
#define DEFAULT_KSZ9897_ADDR	0x5f

/* return 0 on success, -1 on failure */
static int ksz9897_write(int fd, unsigned int addr, unsigned int reg,
			uint32_t data, unsigned int len)
{
	struct i2c_rdwr_ioctl_data msg_rdwr;
	struct i2c_msg i2cmsg;
	int i;
	unsigned int max_length = 4;
	__u8 wbuf[2 + max_length];

	if(reg > 0xFFFF){
		fprintf(stderr,"Sorry, reg(0x%02X) out of range\n", reg);
		return -1;
	}

	if((len < 1) || (len > max_length)) {
		fprintf(stderr,"Sorry, len(%i) out of range\n", len);
		return -1;
	}

	wbuf[0] = (__u8)((reg & 0xff00) >> 8);
	wbuf[1] = (__u8)(reg);

	for(i = (len - 1); i >= 0; --i) /* copy buf[0..n] -> _buf[1..n+1] */
		wbuf[1 + (i + 1)] = data >> (((len - 1) * 8) - (i * 8));

	msg_rdwr.msgs  = &i2cmsg;
	msg_rdwr.nmsgs = 1;

	i2cmsg.addr  = addr;
	i2cmsg.flags = 0;
	i2cmsg.len   = 2 + len;
	i2cmsg.buf   = wbuf;

	if((i=ioctl(fd, I2C_RDWR, &msg_rdwr)) < 0){
		perror("ioctl()");
		fprintf(stderr,"ioctl returned %d\n",i);
		return -1;
	}

	fprintf(stderr,
		"Wrote 0x%0*x with length %i to ksz9897 at 0x%02x, reg 0x%04x\n",
		(len * 2), data, len, addr, reg);

	return 0;
}

/* return -1 on error, 0 on success */
int ksz9897_read(int fd, unsigned int addr, unsigned int reg,
		uint32_t *data, unsigned int len)
{
	struct i2c_rdwr_ioctl_data msg_rdwr;
	struct i2c_msg i2cmsg[2];
	int i;
	unsigned int max_length = 4;
	__u8 wbuf[2] = { (__u8)((reg & 0xff00) >> 8), (__u8)(reg) };
	__u8 rbuf[max_length];

	if(reg > 0xFFFF){
		fprintf(stderr,"Sorry, reg(0x%04X) out of range\n", reg);
		return -1;
	}

	if((len < 1) || (len > max_length)) {
		fprintf(stderr,"Sorry, len(%i) out of range\n", len);
		return -1;
	}

	msg_rdwr.msgs  = i2cmsg;
	msg_rdwr.nmsgs = 2;

	i2cmsg[0].addr  = addr;
	i2cmsg[0].flags = 0;
	i2cmsg[0].len   = 2;
	i2cmsg[0].buf   = (__u8 *)&wbuf;

	i2cmsg[1].addr  = addr;
	i2cmsg[1].flags = I2C_M_RD;
	i2cmsg[1].len   = len;
	i2cmsg[1].buf   = rbuf;

	if((i = ioctl(fd, I2C_RDWR, &msg_rdwr)) < 0){
		perror("ioctl()");
		fprintf(stderr,"ioctl returned %d\n",i);
		return -1;
	}

	*data = 0;
	for(i = 0; i < (int)len; ++i) /* copy buf[0..n] -> _buf[1..n+1] */
		*data |= (((uint32_t)rbuf[i]) << (((len - 1) * 8) - (i * 8)));

	fprintf(stdout,
		"Read 0x%0*x with length %i from ksz9897 at 0x%02x, reg 0x%04x\n",
		(len * 2), *data, len, addr, reg);

	return 0;
}

int main(int argc, char **argv)
{
	int i;
	uint32_t val;
	unsigned int reg = 0x0000;
	unsigned int len = 4;
	/* filedescriptor and name of device */
	int d;
	char *dn = DEFAULT_I2C_BUS;
	unsigned int addr = DEFAULT_KSZ9897_ADDR;
	int rwmode = 0;
	while((i=getopt(argc,argv,"d:a:r:w:l:h"))>=0){
		switch(i){
		case 'h':
		    fprintf(stderr,"%s [-d dev] [-a addr] [-r reg] [-w val] [-l len]\n",argv[0]);
		    fprintf(stderr,"\tdev : device (default is %s)\n", DEFAULT_I2C_BUS);
		    fprintf(stderr,"\taddr: base address of ksz9897 (default is 0x%02x)\n", DEFAULT_KSZ9897_ADDR);
		    fprintf(stderr,"\treg : full register address (default is 0x%04x)\n", reg);
		    fprintf(stderr,"\tval : write val to ksz9897 (default is reading!)\n");
		    fprintf(stderr,"\tlen : bytes to write (default is %u)\n", len);
		    exit(1);
		    break;
		case 'd':
		    dn=optarg;
		    break;
		case 'a':
		    if(sscanf(optarg,"0x%x",&addr)!=1){
			fprintf(stderr,"Cannot parse '%s' as addrs., example: 0x0a\n",
				optarg);
			exit(1);
		    }
		    break;
		case 'r':
		    if(sscanf(optarg,"0x%x",&reg)!=1){
			fprintf(stderr,"Cannot parse '%s' as register, example: 0x1234\n",
				optarg);
			exit(1);
		    }
		    break;
		case 'w':
		    rwmode++;
		    if(sscanf(optarg,"0x%x",&val)!=1){
			fprintf(stderr,"Cannot parse '%s' as value, example: 0x12345678\n",
				optarg);
			exit(1);
		    }
		    break;
		case 'l':
		    if(sscanf(optarg,"%d",&len)!=1){
			fprintf(stderr,"Cannot parse '%s' as value, example: 4\n",
				optarg);
			exit(1);
		    }
		    break;
		}
	}

	fprintf(stderr,"address of ksz9897        : 0x%02x\n",addr);
	fprintf(stderr,"register                  : 0x%04x\n", reg);
	fprintf(stderr,"length                    : %i\n", len);

	if((d = open(dn, O_RDWR)) < 0){
		fprintf(stderr,"Could not open i2c at %s\n",dn);
		perror(dn);
		exit(1);
	}

	fprintf(stderr,"i2c-devicenode is         : %s\n",dn);
	fprintf(stderr,"        on filedescriptor : %d\n\n",d);

	/***
	*** I'm not the one to blame of you screw your switch!
	***/

	if(rwmode){
		if(ksz9897_write(d, addr, reg, val, len) < 0) {
			close(d);
			exit(1);
		}
	} else {
		if(ksz9897_read(d, addr, reg, &val, len) < 0) {
			close(d);
			exit(1);
		}
		fprintf(stdout,"0x%0*x\n", (len * 2), val);
	}

	close(d);
	exit(0);
}
