/*
 * fb16 - small tool for 16 bit  RGB565 framebuffer
 *
 * Copyright (C) 2013-2017 Markus Niebel <Markus.Niebel@tq-group.com>, TQ Systems GmbH
 *
 * License:     GPLv2
 *
 *******************************************************************************
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 ******************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <errno.h>
#include  <unistd.h>
#include <fcntl.h>
#include <linux/fb.h>
#include <sys/mman.h>
#include <sys/ioctl.h>

static void pabort(const char *s, int err)
{
	perror(s);
	exit(err);
}

struct screen_data
{
	struct fb_var_screeninfo vinfo;
	struct fb_fix_screeninfo finfo;
	long int screen_size;	//Number of bytes in the screen
	int fbfd;
	char *fbp;
	char *back_buffer;
};	

/*
 * taken from fbtest,
 * https://git.kernel.org/pub/scm/linux/kernel/git/geert/fbtest.git
 */
void clear_screen(struct screen_data *screen)
{
	uint32_t size = screen->screen_size/sizeof(unsigned long);
	unsigned long *p = (unsigned long *)screen->fbp;

	while (size--)
		*p++ = 0;
}

int init_screen(struct screen_data *screen)
{
	int fbfd = 0;
	long int screen_size = 0;
	struct fb_var_screeninfo vinfo;
	struct fb_fix_screeninfo finfo;
	char *fbp = 0;

	/* Open the file for reading and writing */
	fbfd = open("/dev/fb0", O_RDWR);
	if (!fbfd)
		pabort("Error: cannot open framebuffer device.\n", errno);

	/* Get fixed screen information */
	if (ioctl(fbfd, FBIOGET_FSCREENINFO, &finfo))
		pabort("Error reading fixed information.\n", errno);

	/* Get variable screen information */
	if (ioctl(fbfd, FBIOGET_VSCREENINFO, &vinfo))
		pabort("Error reading variable information.\n", errno);

	/* Figure out the size of the screen in bytes */
	screen_size = vinfo.xres * vinfo.yres * vinfo.bits_per_pixel / 8;
		
	/* Map the device to memory */
	fbp = (char *)mmap(0, screen_size, PROT_READ | PROT_WRITE, MAP_SHARED, fbfd, 0);
	if ((ssize_t)fbp == -1)
		pabort("Error: failed to map framebuffer device to memory.\n", errno); 

	printf("The framebuffer device was mapped to memory successfully.\n");
	if ((int)vinfo.bits_per_pixel != 16) {
		printf("The frambuffer's bit depth is %d.\n", vinfo.bits_per_pixel);
		exit(5);
	}

	//Make sure the xoffset and yoffset are zero...
	if (vinfo.xoffset != 0 || vinfo.yoffset != 0)
		pabort("Error: The frambuffer's x and y offsets are not zero..\n", 42);

	//Store our useful screen stuff in the struct.
	screen->finfo = finfo;
	screen->vinfo = vinfo;
	screen->fbp = fbp;
	screen->fbfd = fbfd;
	screen->screen_size = screen_size;

	//Allocate some memory for the backbuffer.
	screen->back_buffer = malloc(screen_size);

	clear_screen(screen);

	printf("xres: %d, yres: %d, bpp: %d\n", screen->vinfo.xres, screen->vinfo.yres, screen->vinfo.bits_per_pixel);
	printf("xoffset: %d, yoffset: %d\n", screen->vinfo.xoffset, screen->vinfo.yoffset);

	return 0;
}

//This closes the framebuffer device.
int shutdown_screen(struct screen_data *screen)
{
	munmap(screen->fbp, screen->screen_size);
	close(screen->fbfd);
	
	free(screen->back_buffer);
	return 0;
}

void grey_screen(struct screen_data *sd)
{
	size_t x, y, s, v, l;
	uint16_t* p = (uint16_t*)sd->fbp;
	
	for (y = 0; y < sd->vinfo.yres; ++y)
	{
		s = y % (1 << 5);
		printf("line: %zu, start: %zu", y, s);
		for (x = 0; x < sd->vinfo.xres; ++x)
		{
			v = (s << sd->vinfo.red.offset) | 
				(s << (sd->vinfo.green.offset + 1)) | 
				(s << sd->vinfo.blue.offset);
			l = (x + sd->vinfo.xoffset) * (sd->vinfo.bits_per_pixel / 8) + 
			(y + sd->vinfo.yoffset) * sd->finfo.line_length;
			p = (uint16_t*)(&sd->fbp[l]);
			if (0 == x)
			{
				printf(" val[0] = %zx, loc %zu\n", v, l);
			}
			*p = v;
			++s;
			s %= (1 << 5);
		}
	}
}

void fill_screen(struct screen_data *sd, u_int16_t val)
{
	size_t x, y, l;
	uint16_t* p = (uint16_t*)sd->fbp;

	printf("fil: %x\n", val);
	for (y = 0; y < sd->vinfo.yres; ++y)
	{
		printf("p %p\n", p);
		for (x = 0; x < sd->vinfo.xres; ++x)
		{
			l = (x + sd->vinfo.xoffset) * (sd->vinfo.bits_per_pixel / 8) + 
			(y + sd->vinfo.yoffset) * sd->finfo.line_length;
			p = (uint16_t*)(&sd->fbp[l]);
			*p = val;
		}
	}
}


void color_screen(struct screen_data *sd, size_t bits, size_t offset)
{
	size_t x, y, s, v, l;
	uint16_t* p = (uint16_t*)sd->fbp;

	for (y = 0; y < sd->vinfo.yres; ++y)
	{
		s = y % (1 << bits);
		printf("line: %zu, start: %zu", y, s);
		for (x = 0; x < sd->vinfo.xres; ++x)
		{
			v = s << offset;
			l = (x + sd->vinfo.xoffset) * (sd->vinfo.bits_per_pixel / 8) + 
			(y + sd->vinfo.yoffset) * sd->finfo.line_length;
			p = (uint16_t*)(&sd->fbp[l]);
			if (0 == x)
			{
				printf(" val[0] = %zx, loc %zu\n", v, l);
			}
			*p = v;
			++s;
			s %= (1 << bits);
		}
	}
}

void color_bar(struct screen_data *sd)
{
	size_t x, y, s, v, l, w, m;
	uint16_t* p = (uint16_t*)sd->fbp;

	size_t ye = ((((1 << sd->vinfo.red.length) - 1) << sd->vinfo.red.offset) | 
		(((1 << sd->vinfo.green.length) - 1) << sd->vinfo.green.offset));
		
	m = (1 << sd->vinfo.red.length);
	w = sd->vinfo.xres / m;
	printf("w = %zu val = %zu\n", w, m);
	for (y = 0; y < sd->vinfo.yres / 4; ++y)
	{
		for (x = 0; x < sd->vinfo.xres; ++x)
		{
			s = x / w;
			v = (s < m) ? s << sd->vinfo.red.offset : ye;
			l = (x + sd->vinfo.xoffset) * (sd->vinfo.bits_per_pixel / 8) + 
			(y + sd->vinfo.yoffset) * sd->finfo.line_length;
			p = (uint16_t*)(&sd->fbp[l]);
			*p = v;
		}
	}

	m = (1 << sd->vinfo.blue.length);
	w = sd->vinfo.xres / m;
	printf("w = %zu val = %zu\n", w, m);
	for (y = sd->vinfo.yres / 4; y < sd->vinfo.yres / 2; ++y)
	{
		for (x = 0; x < sd->vinfo.xres; ++x)
		{
			s = x / w;
			v = (s < m) ? s << sd->vinfo.blue.offset : ye;
			l = (x + sd->vinfo.xoffset) * (sd->vinfo.bits_per_pixel / 8) + 
			(y + sd->vinfo.yoffset) * sd->finfo.line_length;
			p = (uint16_t*)(&sd->fbp[l]);
			*p = v;
		}
	}

	m = (1 << sd->vinfo.green.length);
	w = sd->vinfo.xres / m;
	printf("w = %zu val = %zu\n", w, m);
	for (y = sd->vinfo.yres / 2; y < sd->vinfo.yres / 4 * 3; ++y)
	{
		for (x = 0; x < sd->vinfo.xres; ++x)
		{
			s = x / w;
			v = (s < m) ? s << sd->vinfo.green.offset : ye;
			l = (x + sd->vinfo.xoffset) * (sd->vinfo.bits_per_pixel / 8) + 
			(y + sd->vinfo.yoffset) * sd->finfo.line_length;
			p = (uint16_t*)(&sd->fbp[l]);
			*p = v;
		}
	}

	m = (1 << sd->vinfo.blue.length);
	w = sd->vinfo.xres / m;
	for (y = sd->vinfo.yres / 4 * 3; y < sd->vinfo.yres; ++y)
	{
		for (x = 0; x < sd->vinfo.xres; ++x)
		{
			s = x / w;
			v = (s << sd->vinfo.red.offset) | 
				(s << (sd->vinfo.green.offset + 1)) | 
				(s << sd->vinfo.blue.offset);
			l = (x + sd->vinfo.xoffset) * (sd->vinfo.bits_per_pixel / 8) + 
				(y + sd->vinfo.yoffset) * sd->finfo.line_length;
			p = (uint16_t*)(&sd->fbp[l]);
			*p = v;
		}
	}

}

void color_frame(struct screen_data *sd)
{
	size_t x, y, l, c;
	uint16_t* p;
	
	c = (1 << sd->vinfo.green.length) - 1;
	c <<= sd->vinfo.green.offset;
	for (x = 0; x < sd->vinfo.xres; ++x)
	{
		l = (x + sd->vinfo.xoffset) * (sd->vinfo.bits_per_pixel / 8) + 
		(0 + sd->vinfo.yoffset) * sd->finfo.line_length;
		p = (uint16_t*)(&sd->fbp[l]);
		*p = c;
	}
	c = ((((1 << sd->vinfo.red.length) - 1) << sd->vinfo.red.offset) | 
		(((1 << sd->vinfo.green.length) - 1) << sd->vinfo.green.offset));
	for (x = 0; x < sd->vinfo.xres; ++x)
	{
		l = (x + sd->vinfo.xoffset) * (sd->vinfo.bits_per_pixel / 8) + 
		(sd->vinfo.yres - 1 + sd->vinfo.yoffset) * sd->finfo.line_length;
		p = (uint16_t*)(&sd->fbp[l]);
		*p = c;
	}
	c = (((1 << sd->vinfo.red.length) - 1) << sd->vinfo.red.offset);
	for (y = 0; y < sd->vinfo.yres; ++y)
	{
		l = (0 + sd->vinfo.xoffset) * (sd->vinfo.bits_per_pixel / 8) + 
			(y + sd->vinfo.yoffset) * sd->finfo.line_length;
		p = (uint16_t*)(&sd->fbp[l]);
		*p = c;
	}
	c = (((1 << sd->vinfo.blue.length) - 1) << sd->vinfo.blue.offset);
	for (y = 0; y < sd->vinfo.yres; ++y)
	{
		l = (sd->vinfo.xres - 1 + sd->vinfo.xoffset) * (sd->vinfo.bits_per_pixel / 8) + 
			(y + sd->vinfo.yoffset) * sd->finfo.line_length;
		p = (uint16_t*)(&sd->fbp[l]);
		*p = c;
	}
}

int main(int argc, char *argv[])
{
	struct screen_data sd;
	if (0 == init_screen(&sd))
	{
		
		if (1 == argc) {
			grey_screen(&sd);
		} else if ( 0 == strcmp(argv[1], "--clear")) {
			clear_screen(&sd);
		} else if ( 0 == strcmp(argv[1], "--red")) {
			color_screen(&sd, sd.vinfo.red.length, sd.vinfo.red.offset);
		} else if ( 0 == strcmp(argv[1], "--green")) {
			color_screen(&sd, sd.vinfo.green.length, sd.vinfo.green.offset);
		} else if ( 0 == strcmp(argv[1], "--blue")) {
			color_screen(&sd, sd.vinfo.blue.length, sd.vinfo.blue.offset);
		} else if ( 0 == strcmp(argv[1], "--bar")) {
			color_bar(&sd);
		} else if ( 0 == strcmp(argv[1], "--frame")) {
			color_frame(&sd);
		} else if ( 0 == strcmp(argv[1], "--fill")) {
			unsigned long val = 0xf00f;
			if (argc > 2) {
				val = strtoul(argv[2], 0, 16);
			}
			fill_screen(&sd, val);
		} else if ( 0 == strcmp(argv[1], "--help")) {
			printf("%s - a framebuffer test for RGB 565\n", argv[0]);
			printf("(C) TQ Systems GmbH 2010 - 2015\n");
			printf("usage:\n");
			printf("%s --help\n", argv[0]);
			printf("%s ........... grey stripes\n", argv[0]);
			printf("%s --clear.... clear screen\n", argv[0]);
			printf("%s --red ..... red stripes\n", argv[0]);
			printf("%s --green ... green stripes\n", argv[0]);
			printf("%s --blue .... blue stripes\n", argv[0]);
			printf("%s --bar ..... color gradient bars\n", argv[0]);
			printf("%s --frame ... frame at the boarder of the display\n", argv[0]);
			printf("%s --fill .... fill with solid color\n", argv[0]);
		}
	
		
		shutdown_screen(&sd);
	}
	return 0;
}

