/*
 * fbclr - small tool for 32 bit  RGB8880 framebuffer
 *
 * Copyright (C) 2017 Markus Niebel <Markus.Niebel@tq-group.com>, TQ Systems GmbH
 *
 * License:     GPLv2
 *
 *******************************************************************************
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 ******************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <errno.h>
#include  <unistd.h>
#include <fcntl.h>
#include <linux/fb.h>
#include <sys/mman.h>
#include <sys/ioctl.h>

enum mode {
	mode_help,
	mode_fill,
	mode_clear,
};

static void pabort(const char *s, int err)
{
	perror(s);
	exit(err);
}

struct screen_data
{
	struct fb_var_screeninfo vinfo;
	struct fb_fix_screeninfo finfo;
	long int screen_size;	//Number of bytes in the screen
	int fbfd;
	char *fbp;
};

void show_usage(const char *progname)
{
	printf("%s - clear framebuffer\n", progname);
	printf("(C) TQ Systems GmbH 2017\n");
	printf("usage:\n");
	printf("%s --help\n", progname);
	printf("%s --clear............ clear screen\n", progname);
	printf("%s --fill <clolor>.... fill with solid color\n", progname);
}

int init_screen(struct screen_data* screen)
{
	int fbfd = 0;
	long int screen_size = 0;
	struct fb_var_screeninfo vinfo;
	struct fb_fix_screeninfo finfo;
	char *fbp = 0;

	/* Open the file for reading and writing */
	fbfd = open("/dev/fb0", O_RDWR);
	if (!fbfd)
		pabort("Error: cannot open framebuffer device.\n", errno);

	/* Get fixed screen information */
	if (ioctl(fbfd, FBIOGET_FSCREENINFO, &finfo))
		pabort("Error reading fixed information.\n", errno);

	/* Get variable screen information */
	if (ioctl(fbfd, FBIOGET_VSCREENINFO, &vinfo))
		pabort("Error reading variable information.\n", errno);

	/* Figure out the size of the screen in bytes */
	screen_size = vinfo.xres * vinfo.yres * vinfo.bits_per_pixel / 8;
		
	/* Map the device to memory */
	fbp = (char *)mmap(0, screen_size, PROT_READ | PROT_WRITE, MAP_SHARED, fbfd, 0);
	if ((ssize_t)fbp == -1)
		pabort("Error: failed to map framebuffer device to memory.\n", errno); 

	//Make sure the xoffset and yoffset are zero...
	if (vinfo.xoffset != 0 || vinfo.yoffset != 0)
		pabort("Error: The frambuffer's x and y offsets are not zero..\n", 42);

	//Store our useful screen stuff in the struct.
	screen->finfo = finfo;
	screen->vinfo = vinfo;
	screen->fbp = fbp;
	screen->fbfd = fbfd;
	screen->screen_size = screen_size;

	return 0;
}

//This closes the framebuffer device.
int shutdown_screen(struct screen_data *screen)
{
	munmap(screen->fbp, screen->screen_size);
	close(screen->fbfd);
	
	return 0;
}

/*
 * taken from fbtest,
 * https://git.kernel.org/pub/scm/linux/kernel/git/geert/fbtest.git
 */
void clear_screen(struct screen_data *screen)
{
	uint32_t size = screen->screen_size/sizeof(unsigned long);
	unsigned long *p = (unsigned long *)screen->fbp;

	while (size--)
		*p++ = 0;
}

void fill_screen(struct screen_data* screen, u_int32_t val)
{
	switch ((screen->vinfo.bits_per_pixel + 7) / 8) {
	case 1:
	{
		uint8_t* p = (uint8_t*)screen->fbp;
		size_t size = screen->screen_size/sizeof(uint8_t);

		while (size--)
			*p++ = val;
		break;
	}
	case 2:
	{
		uint16_t* p = (uint16_t*)screen->fbp;
		size_t size = screen->screen_size/sizeof(uint16_t);

		while (size--)
			*p++ = val;
		break;
	}
	case 4:
	{
		uint32_t* p = (uint32_t*)screen->fbp;
		size_t size = screen->screen_size/sizeof(uint32_t);

		while (size--)
			*p++ = val;
		break;
	}
	default:
		pabort("unsupported pixel depth", EINVAL);
	}
}

int main(int argc, char *argv[])
{
	struct screen_data sd;
	enum mode action = mode_help;
	unsigned long val = 0x00ffffff;
	int ret = 0;

	if (1 == argc) {
		action = mode_clear;
	} else if ((2 == argc) && (0 == strcmp(argv[1], "--clear"))) {
		action = mode_clear;
	} else if ( 0 == strcmp(argv[1], "--fill")) {
		action = mode_fill;
		if (argc == 3) {
			val = strtoul(argv[2], 0, 16);
		}
	} else if ( 0 == strcmp(argv[1], "--help")) {
		action = mode_help;
	} else {
		action = mode_help;
		ret = EINVAL;
	}

	if (ret || (action == mode_help)) {
		show_usage(argv[0]);
	} else {
		if (0 == init_screen(&sd)) {
			switch (action) {
			case mode_clear:
				clear_screen(&sd);
				break;
			case mode_fill:
				fill_screen(&sd, val);
				break;
			default:
				show_usage(argv[0]);
				ret = EINVAL;
			}
			shutdown_screen(&sd);
		}
	}

	return ret;
}
