/*
 * Watchdog Driver Test Program
 */

#include <stdint.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdbool.h>
#include <errno.h>
#include <getopt.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/types.h>
#include <linux/watchdog.h>

enum wdt_cmd {
	wdt_none,
	wdt_kill,
	wdt_show,
	wdt_timeout,
	wdt_write,
	wdt_keep,
};

struct wdt_config {
	const char *device;
	int fd;
	enum wdt_cmd cmd;
	unsigned timeout;
};

static void pabort(const char *s, int err)
{
	perror(s);
	exit(err);
}

/*
 * This function simply sends an IOCTL to the driver, which in turn triggers
 * the CPU watchdog to reset its internal timer so it doesn't trigger a board
 * reset.
 */
void keep_alive(struct wdt_config *cfg)
{
	int dummy;

	int r = ioctl(cfg->fd, WDIOC_KEEPALIVE, &dummy);
	if (r == -1)
		pabort("WDIOC_KEEPALIVE failed", errno);
}

static void print_usage(const char *prog)
{
	printf("Usage: %s [-dsbcho]\n", prog);
	puts("  -d --device          device to use (optional, default /dev/watchdog0\n"
	     "  -k --kill            disable watchdog\n"
	     "  -s --show            show watchdog timeout\n"
	     "  -t --timeout <val>   set watchdog timeout\n"
	     "  -w --write <val>     cyclical write\n"
	     "  -a --alive <val>     cyclical keep alive ioctl\n");

	exit(1);
}

static int parse_opts(int argc, char *argv[], struct wdt_config *config)
{
	static const struct option lopts[] = {
		{ "device",  1, 0, 'd' },
		{ "kill",    0, 0, 'k' },
		{ "show",    0, 0, 's' },
		{ "timeout", 1, 0, 't' },
		{ "write",   1, 0, 'w' },
		{ "alive",   1, 0, 'a' },
		{ NULL,      0, 0,  0 },
	};
	int c;

	while (1) {
		c = getopt_long(argc, argv, "d:skwt:", lopts, NULL);

		if (c == -1)
			break;

		switch (c) {
		case 'd':
			config->device = optarg;
			break;
		case 'k':
			config->cmd = wdt_kill;
			break;
		case 's':
			config->cmd = wdt_show;
			break;
		case 't':
		case 'a':
		case 'w':
			errno = 0;
			config->timeout = strtoul(optarg, 0, 10);
			if (errno) {
				fprintf(stderr, "%s: -a needs an unsigned number\n",
					argv[0]);
				print_usage(argv[0]);
				return EINVAL;
			}
			switch (c) {
			case 't':
				config->cmd = wdt_timeout;
				break;
			case 'a':
				config->cmd = wdt_keep;
				break;
			case 'w':
				config->cmd = wdt_write;
				break;
			}
			break;
		default:
			print_usage(argv[0]);
			return -1;
			break;
		}
	}
	return 0;
}


/*
 * The main program. Call with --help to display all possible options
 */
int main(int argc, char *argv[])
{
	struct wdt_config config = {
		.device = "/dev/watchdog0",
		.fd = -1,
		.cmd = wdt_none,
	};
	int timeout;
	int ret;
	if (parse_opts(argc, argv, &config))
		return EINVAL;

	if (wdt_none == config.cmd) {
		print_usage(argv[0]);
		return -1;
	}

	config.fd = open(config.device, O_WRONLY);
	if (config.fd < 0)
		pabort("can't open device", errno);

	switch(config.cmd) {
	case wdt_kill:
		ret = write(config.fd, "V", 1);
		fprintf(stderr, "Watchdog disabled.\n");
		fflush(stderr);
		exit(0);
		break;
	case wdt_show:
		ioctl(config.fd, WDIOC_GETTIMEOUT, &timeout);
		fprintf(stderr, "Current watchdog timeout = %d s\n",
	 		timeout);
		fflush(stderr);
		exit(0);
		break;
	case wdt_timeout:
		ioctl(config.fd, WDIOC_SETTIMEOUT, &config.timeout);
		fprintf(stderr, "Set watchdog timeout to = %d s\n",
			config.timeout);
		fflush(stderr);
		exit(0);
		break;
	case wdt_write:
		while (1) {
			ret = write(config.fd, "\0", 1);
			printf("watchdog triggered\n");
			if (ret != 1) {
				ret = -1;
				break;
			}
			sleep(config.timeout);
		}
		break;
	case wdt_keep:
		while (1) {
			keep_alive(&config);
			sleep(config.timeout);
		}
		break;
	}

	close(config.fd);

	return -33;
}
