#!/bin/bash
# SPDX-License-Identifier: GPL-2.0-or-later
#
# File:        create-mirror-archive.sh
#
# Copyright (C) 2018-2022 TQ-Systems GmbH <oss@ew.tq-group.com>, D-82229 Seefeld, Germany.
# Author: Markus Niebel
#
###############################################################################

# DEBUGGING
# set -e 
set -C # noclobber

# RETURN VALUES/EXIT STATUS CODES
readonly E_BAD_OPTION=254
readonly E_UNKNOWN=255

readonly PROGRAM="$(basename ${0})"
readonly SCRIPT_LIBDIR="$(dirname ${0})"

readonly LIC_EXCLUDE_LIST="ignore proprietary unknown"

PLATFORM_RELEASE_STAMP=
PLATFORM_VERSION_PART=
PLATFORM_PLATFORM_PART=
HAVE_GIT="0"
PLATFORM_MIRROR_ARCHIVE=/dev/null

source "${SCRIPT_LIBDIR}/tools_lib_ptxdist.sh"
source "${SCRIPT_LIBDIR}/tools_lib_git.sh"

PTXDIST=ptxdist

function usage () {
    echo "Usage is as follows:"
    echo
    echo "$PROGRAM <--usage|--help|-?>"
    echo "    Prints this usage output and exits."
    echo "$PROGRAM"
    echo "    compile license report using ptxdist."
    echo "$PROGRAM --stamp <name>"
    echo "    compile license report using ptxdist from <stamp> if under git."
    echo
}

function do_mirror_archive () {
	local VERSION=$1
	local PLATFORM=$2
	local PLATFORM_DIST_DIR=${PLATFORM}/dist

	local MIRROR_DATA_DIR=${PLATFORM}/mirror

	rm -rf ${MIRROR_DATA_DIR}

	${PTXDIST} get
	ret=$?
	[ "${?}" -ne "0" ] && echo "download failed. please verify result of ${PTXDIST} get" && exit $1;

	ptxdist_has_working_print ${VERSION}
	[ "${?}" -ne "0" ] && echo "${PTXDIST} don't support print, cannot create mirror archive" && exit 1;

	mkdir -p ${MIRROR_DATA_DIR}

	local pkglist=$(${PTXDIST} print PTX_PACKAGES_SELECTED)

	for p in ${pkglist}; do
		local name=$(echo "${p}" | tr '[:lower:]' '[:upper:]');
		name=$(echo "${name}" | tr '-' '_');
		echo "pkg $p name $name"
		local srcvar=${name}_SOURCE;
		local licvar=${name}_LICENSE;
		local src=$(${PTXDIST} print $srcvar 2>/dev/null);
		local lic=$(${PTXDIST} print $licvar 2>/dev/null);

		if ! [ -z "$lic" ] && ! [ - z "$src" ] ; then

			echo "pkg $p src $src lic $lic"
			echo ${LIC_EXCLUDE_LIST} | grep -w $lic 1>/dev/null 2>/dev/null
			[ "$?" -ne "0" ] && [ -f $src ] && cp $src ${MIRROR_DATA_DIR}

		fi
	done;

	tar -cvzf ${PLATFORM_MIRROR_ARCHIVE} -C $MIRROR_DATA_DIR .
}

function main () {
	# Process command-line arguments.
	while test $# -gt 0; do
	    case $1 in
		--stamp )
		    shift
		    STAMP="$1"
		    shift
		    ;;

		-? | --usage | --help )
		    usage
		    exit
		    ;;

		-* )
		    echo "Unrecognized option: $1" >&2
		    usage
		    exit $E_BAD_OPTION
		    ;;

		* )
		    break
		    ;;
	    esac
	done

	[ check_git ] && HAVE_GIT="1"

	check_ptxdist

	local CONFIG_FILE_VERSION=$(get_ptxconfig_version)
	local PLATFORM_FILE_VERSION=$(get_platformconfig_version)

	if [ "x${CONFIG_FILE_VERSION}" != "x${PLATFORM_FILE_VERSION}" ]; then
		echo "platform and config versions differ! Please correct.";
		exit $E_UNKNOWN;
	fi

	check_ptxdist_version ${CONFIG_FILE_VERSION}
	PTXDIST=ptxdist-${CONFIG_FILE_VERSION}

	local PLATFORM_NAME=$(get_platform_name)
	echo "using ${PLATFORM_NAME}"

	local PLATFORM=platform-${PLATFORM_NAME}
	echo "using ${PLATFORM}"

	local PLATFORM_DIST_DIR=${PLATFORM}/dist
	mkdir -p ${PLATFORM_DIST_DIR}

	do_query_release_type ${CONFIG_FILE_VERSION} ${PLATFORM} ${STAMP}

	PLATFORM_MIRROR_ARCHIVE=${PLATFORM_DIST_DIR}/${PLATFORM_PLATFORM_PART}.Mirror.SRC.${PLATFORM_VERSION_PART}.tar.gz

	do_mirror_archive ${CONFIG_FILE_VERSION} ${PLATFORM}

	return $ret
}

main $@

