#!/bin/sh
# SPDX-License-Identifier: GPL-2.0-or-later
#
# File:        gentree.sh
#
# Copyright (C) 2015-2022 TQ-Systems GmbH <oss@ew.tq-group.com>, D-82229 Seefeld, Germany.
# Author: Markus Niebel
#
# Description: A utility script that configures a patch stack for a BSP.
#
# Examples:    gentree.sh --verbose --config=<file>
#              gentree.sh --help
#
###############################################################################

# DEBUGGING
# set -e
set -C # noclobber

# =============================================================================
#
# =============================================================================

# RETURN VALUES/EXIT STATUS CODES
readonly E_BAD_OPTION=254
readonly E_UNKNOWN=255

verbose=0
readonly PROGRAM=${0}

DEST=none
PROJECT=""
BASEREV=""
BRANCHES=""
CONFIGFILE=""

log() {
	if [ ${verbose} -ne "0" ]
	then
		echo "LOG: ${1}"
	fi
}

err() {
	echo "error: $@" >&2
	exit 5
}

do_usage() {
	echo "Usage is as follows:"
	echo
	echo "$PROGRAM <--usage|--help|-?>"
	echo "    Prints this usage output and exits."
	echo "$PROGRAM [--verbose n] --config <file>"
	echo "    start with config from <file>"
	echo "    and optionally be verbose"
	echo "the config file must contain the following declarations:"
	echo "readonly GENTREE_BASEREV=<git baserev>"
	echo "readonly GENTREE_BRANCHES=<ordered list of branch names>"
	echo "readonly GENTREE_PROJECT=<project name>"
	echo "readonly GENTREE_REMOTE=<remote name>"
}

# Process command-line arguments.
parse_options() {
	while test ${#} -gt 0; do
		case ${1} in
			--verbose )
				shift
				verbose="${1}"
				shift
				;;
			--config )
				shift
				CONFIGFILE="${1}"
				shift
				;;
			-? | --usage | --help )
				do_usage
				exit
				;;
			-* )
				do_usage
				err "Unrecognized option: $1"
				;;
			* )
				break
				;;
		esac
	done
}

###
# source_config <file>
###
source_config() {
	local CONFIGFILE=$(readlink -f "${1}")
	[ -f ${CONFIGFILE} ] || err "${CONFIGFILE} not existing or not a file"
	[ -s ${CONFIGFILE} ] || err "${CONFIGFILE} empty"

	log "sourcing ${CONFIGFILE}"
	. "${CONFIGFILE}"
	BASEREV=${GENTREE_BASEREV}
	PROJECT=${GENTREE_PROJECT}
	BRANCHES=${GENTREE_BRANCHES}
	REMOTE=$(git config --get remote.${GENTREE_REMOTE}.url)
	[ -z "${BASEREV}" ] && err "BASEREV is not set"
	[ -z "${BRANCHES}" ] && err "BRANCHES is not set"
	[ -z "${PROJECT}" ] && err "PROJECT is not set"
	[ -z "${REMOTE}" ] && err "REMOTE is not set"

	DEST="${BASEREV}/${PROJECT}-$(date +"%Y%m%d")"
}

cleanup_branches() {
	for b in ${BRANCHES}
	do
		test=$(git branch | grep ${b})
		exist=${?}
		if [ "${exist}" -eq "0" ]
		then
			log "delete merging/${b}"
			git branch -D merging/${b}
		fi
	done
}

fetch_branches() {
	for b in ${BRANCHES}
	do
		git fetch ${REMOTE} ${b}:merging/${b}
		if [ "${?}" -ne "0" ]
		then
			err "fetching ${b} from ${REMOTE}"
			exit 2
		fi
	done
}

merge_branches() {
	for b in ${BRANCHES}
	do
		git checkout merging/${b}
		git rebase ${DEST}
		if [ "${?}" -ne "0" ]
		then
			err "rebase ${b}"
		fi
		git checkout ${DEST}
		git merge --stat --log merging/${b}
		if [ "${?}" -ne "0" ]
		then
			err "merging ${b}"
		fi
	done
}

main() {
	log "read config ..."

	parse_options $@
	source_config ${CONFIGFILE}

	log "prepare ..."

	git branch -D ${DEST}
	cleanup_branches

	log "fetch ..."
	fetch_branches
	if [ "${?}" -ne "0" ]
	then
		cleanup_branches
	fi

	log "checkout ..."
	git checkout -b ${DEST} ${BASEREV}
	if [ "${?}" -ne "0" ]
	then
		err "creating ${DEST} from ${BASEREV}"
		cleanup_branches
	fi

	log "merge ..."
	merge_branches

	log "completed ..."
}

main ${@}

