#!/bin/bash
# SPDX-License-Identifier: GPL-2.0-or-later
#
# File:        install_ptxdist.sh
#
# Copyright (C) 2016-2022 TQ-Systems GmbH <oss@ew.tq-group.com>, D-82229 Seefeld, Germany.
# Author: Markus Niebel
#
# Description: A utility script to install a PTXdist version
#
###############################################################################

# DEBUGGING
# set -e
set -C # noclobber

VERSION=
VERBOSE=0

# RETURN VALUES/EXIT STATUS CODES
readonly E_BAD_OPTION=254
readonly E_UNKNOWN=255
readonly BASE_URL=http://ptxdist.org/software/ptxdist/download

function usage () {
    echo "Usage is as follows:"
    echo
    echo "$PROGRAM <--usage|--help|-?>"
    echo "    Prints this usage output and exits."
    echo "$PROGRAM --version"
    echo "    install this version of ptxdist"
    echo
}

function log () {
    if [ $VERBOSE -eq 1 ]; then
        echo $1
    fi
}

function error () {
    echo $1 >&2
}

function main () {
	# Process command-line arguments.
	while test $# -gt 0; do
	    case $1 in

		--version )
		    shift
		    VERSION=$1
		    shift
		    ;;

		--verbose )
		    shift
		    VERBOSE=1
		    ;;

		-? | --usage | --help )
		    usage
		    exit
		    ;;

		-* )
		    error "Unrecognized option: $1" 
		    usage
		    exit $E_BAD_OPTION
		    ;;

		* )
		    break
		    ;;
	    esac
	done

	if [ -z ${VERSION} ]
	then
		error "missing version"
		usage
		exit $E_BAD_OPTION
	fi

	PTXDIST=ptxdist-${VERSION}
	mkdir -p ./build-${PTXDIST}
	cd build-${PTXDIST}
	if [ $? -ne 0 ]; then
		error "failed to create build dir"
		exit $E_BAD_OPTION
	fi
	rm -f "${PTXDIST}.tar.bz2"
	wget "${BASE_URL}/${PTXDIST}.tar.bz2"
	if [ ! -f "${PTXDIST}.tar.bz2" ]; then
		error "failed to download ${PTXDIST}.tar.bz2"
		exit $E_BAD_OPTION
	fi
	tar -xjf "${PTXDIST}.tar.bz2"
	cd "${PTXDIST}"
	if [ $? -ne 0 ]; then
		error "failed to extract ${PTXDIST}"
		exit $E_BAD_OPTION
	fi
	./configure
	if [ ! -f Makefile ]; then
		error "failed to configure ${PTXDIST}"
		exit $E_BAD_OPTION
	fi
	make
	sudo make install
	TESTVERSION="$(cd / && /usr/local/bin/${PTXDIST} --version)"
	if [ "${TESTVERSION}" = "${VERSION}" ]; then
		echo "installed ${PTXDIST}"
	else
		error "failed to install ${PTXDIST}"
		exit $E_BAD_OPTION
	fi
}

main $@


