#!/bin/bash
# SPDX-License-Identifier: GPL-2.0-or-later
#
# File:        license.sh
#
# Copyright (C) 2015-2022 TQ-Systems GmbH <oss@ew.tq-group.com>, D-82229 Seefeld, Germany.
# Author: Markus Niebel
#
# Description: A utility script that extracts license info from the BSP.
#
###############################################################################

# DEBUGGING
# set -e 
set -C # noclobber

# RETURN VALUES/EXIT STATUS CODES
readonly E_BAD_OPTION=254
readonly E_UNKNOWN=255

readonly PROGRAM="$(basename ${0})"
readonly SCRIPT_LIBDIR="$(dirname ${0})"

PLATFORM_RELEASE_STAMP=
PLATFORM_VERSION_PART=
PLATFORM_PLATFORM_PART=
HAVE_GIT="0"
PLATFORM_LICENSE_ARCHIVE=/dev/null

source "${SCRIPT_LIBDIR}/tools_lib_ptxdist.sh"
source "${SCRIPT_LIBDIR}/tools_lib_git.sh"

PTXDIST=ptxdist

function usage () {
    echo "Usage is as follows:"
    echo
    echo "$PROGRAM <--usage|--help|-?>"
    echo "    Prints this usage output and exits."
    echo "$PROGRAM"
    echo "    compile license report using ptxdist."
    echo "$PROGRAM --stamp <name>"
    echo "    compile license report using ptxdist from <stamp> if under git."
    echo
}


function do_license_report () {
	local VERSION=$1
	local PLATFORM=$2
	local PLATFORM_DIST_DIR=$PLATFORM/dist

	echo "collect license information ...";
	local REPORT_DATA_DIR

	$PTXDIST make license-report
	ret=$?
	if [ $ret -ne 0 ]; then
		echo "license report failed! Please review log.";
		exit $E_UNKNOWN;
	fi

	echo "archive license data ...";
	if [ -d $PLATFORM/state/report ]; then
		REPORT_DATA_DIR=$PLATFORM/state/report
		REPORT_OUT_DIR=$PLATFORM/state
	fi
	if [ -d $PLATFORM/report ]; then
		REPORT_DATA_DIR=$PLATFORM/report
		REPORT_OUT_DIR=$REPORT_DATA_DIR
	fi

	tar -cvzf ${PLATFORM_LICENSE_ARCHIVE} -C $REPORT_DATA_DIR .
	cp $REPORT_OUT_DIR/license-report.pdf $PLATFORM_DIST_DIR/${PLATFORM_PLATFORM_PART}.license-report.${PLATFORM_VERSION_PART}.pdf
	cp $REPORT_OUT_DIR/license-report-tools.pdf $PLATFORM_DIST_DIR/${PLATFORM_PLATFORM_PART}.license-report-tools.${PLATFORM_VERSION_PART}.pdf
}

function do_license_collect () {
	local VERSION=$1
	local PLATFORM=$2
	local PLATFORM_DIST_DIR=$PLATFORM/dist

	local REPORT_DATA_DIR=$PLATFORM/license-collect

	rm -rf $REPORT_DATA_DIR

	$PTXDIST go
	ret=$?
	if [ $ret -ne 0 ]; then
		echo "platform build failed. please verify result of $PTXDIST go";
		exit $ret;
	fi

	ptxdist_has_working_print ${VERSION}
	if [ "${?}" -ne "0" ]; then
		echo "${PTXDIST} don't support print, cannot extract license info";
		exit $ret;
	fi

	mkdir -p $REPORT_DATA_DIR

	local pkglist=$($PTXDIST print PTX_PACKAGES_SELECTED)

	for p in $pkglist; do
		local name=$(echo "$p" | tr '[:lower:]' '[:upper:]');
		name=$(echo "$name" | tr '-' '_');
	##	echo "pkg $p name $name"
		local vervar=${name}_VERSION;
		local ver=$($PTXDIST print $vervar);
		local dirvar=${name}_DIR;
		local pdir=$($PTXDIST print $dirvar);
		mkdir $REPORT_DATA_DIR/$p-$ver;
		local slist="COPYING LICENSE EULA"
		local flist=;
		for f in $slist; do
			if [ -f $pdir/$f ]; then
				flist="$f $flist ";
			fi
		done
## query list of files in dir
## if not empty, get name of file without path only if it is a file
		local lfiles=$(ls -F $pdir/LICENSE.* 2>/dev/null)
		if [ "x$lfiles" != "x" ]; then
			for f in $lfiles; do
				s=$(basename $f | grep -v '[/@=|]');
				flist="$s $flist ";
			done
		fi
## check if license files var is set and add this to our list
		local licfilevar=${name}_LICENSE_FILES;
		local licfiles=$($PTXDIST print $licfilevar);
		if [ "x$licfiles" != "x" ]; then
			flist="$licfiles $flist ";
		fi
## copy all license files
		for f in $flist; do
			cp $pdir/$f $REPORT_DATA_DIR/$p-$ver;
		done
# Fallback: copy README when no license file is found
		local info=$(ls -F $REPORT_DATA_DIR/$p-$ver 2>/dev/null | grep -v '[/@=|]')
		if [ "x$info" == "x" ]; then
			if [ -f $pdir/README ]; then
				cp $pdir/README $REPORT_DATA_DIR/$p-$ver;
			fi
		fi
# license name(s) should be in the state folder ...
		if [ -f $PLATFORM/state/$p.license ]; then
			cp $PLATFORM/state/$p.license $REPORT_DATA_DIR/$p-$ver;
		fi
	done;

	tar -cvzf ${PLATFORM_LICENSE_ARCHIVE} -C $REPORT_DATA_DIR .
}

function main () {
	# Process command-line arguments.
	while test $# -gt 0; do
	    case $1 in
		--stamp )
		    shift
		    STAMP="$1"
		    shift
		    ;;

		-? | --usage | --help )
		    usage
		    exit
		    ;;

		-* )
		    echo "Unrecognized option: $1" >&2
		    usage
		    exit $E_BAD_OPTION
		    ;;

		* )
		    break
		    ;;
	    esac
	done

	[ check_git ] && HAVE_GIT="1"

	check_ptxdist

	local CONFIG_FILE_VERSION=$(get_ptxconfig_version)
	local PLATFORM_FILE_VERSION=$(get_platformconfig_version)

	if [ "x${CONFIG_FILE_VERSION}" != "x${PLATFORM_FILE_VERSION}" ]; then
		echo "platform and config versions differ! Please correct.";
		exit $E_UNKNOWN;
	fi

	check_ptxdist_version ${CONFIG_FILE_VERSION}
	PTXDIST=ptxdist-${CONFIG_FILE_VERSION}

	local PLATFORM_NAME=$(get_platform_name)
	echo "using ${PLATFORM_NAME}"

	local PLATFORM=platform-${PLATFORM_NAME}
	echo "using ${PLATFORM}"

	local PLATFORM_DIST_DIR=$PLATFORM/dist
	mkdir -p $PLATFORM_DIST_DIR

	do_query_release_type ${CONFIG_FILE_VERSION} ${PLATFORM} ${STAMP}

	PLATFORM_LICENSE_ARCHIVE=${PLATFORM_DIST_DIR}/${PLATFORM_PLATFORM_PART}.Licenses.SRC.${PLATFORM_VERSION_PART}.tar.gz

	ptxdist_has_license_report ${CONFIG_FILE_VERSION}
	if [ "${?}" -eq "0" ]; then
		do_license_report ${CONFIG_FILE_VERSION} ${PLATFORM}
		ret=$?
	else
		do_license_collect ${CONFIG_FILE_VERSION} ${PLATFORM}
		ret=$?
	fi

	return $ret
}

main $@

