#!/bin/bash
# SPDX-License-Identifier: GPL-2.0-or-later
#
# File:        maintain.sh
#
# Copyright (C) 2014-2022 TQ-Systems GmbH <oss@ew.tq-group.com>, D-82229 Seefeld, Germany.
# Author: Markus Niebel
#
# Description: A utility script that helps BSP maintenance.
#
###############################################################################

# DEBUGGING
# set -e
set -C # noclobber

VERBOSE=1

readonly PROGRAM="$(basename ${0})"
readonly CONF_SCRIPT_DIR="$(dirname ${0})"

function usage () {
cat <<- EOF
	Usage: ${PROGRAM} [--mode <mode>] [--component <component>] [--version <ver>] [--configs <list>]

	Execute given maintenance mode for all available / given configurations
	(<workspace>/tools/config-* scripts). For all modes a component can be
	given
	
	    mode:
	        default is oldconfig
	        migrate - migrate to ptxdist version given with versions,
	            component is not used
	        oldconfig - do ptxdist oldconfig for given component
	        randconfig - do ptxdist randconfig for given component
	        allyesconfig - do ptxdist allyesconfig for given component
	        allnoconfig - do ptxdist allnoconfig for given component
	        allmodconfig - do ptxdist allmodconfig for given component
	        get - run get stage for all selected packages (component must be empty)
	        urlcheck - run urlcheck for all selected packages (component must be empty)
	        go, images, ... - run build stage (component must be empty)
	    component:
	        default is empty
	        platform - run on platformconfig
	        kernel - run on kernelconfig
	        barebox - run on bareboxconfig
	    version
	        default is empty, only nedded for migrate, use yyyy.mm.n
	    configs
	        default is empty (use all), can be used to narrow selection
	        must be last parameter - a space separated list of target names
	        for tools/config-<name>
EOF
}

# RETURN VALUES/EXIT STATUS CODES
readonly E_BAD_OPTION=254
readonly E_UNKNOWN=255

function log () {
    if [ $VERBOSE -eq 1 ]; then
        echo $1
    fi
}

function error () {
    echo $1 >&2
}

main () {
	local MODE="oldconfig"
	local COMPONENT=""
	local CONFIGS=""

	while test $# -gt 0; do
	    case $1 in
		--mode)
		    shift
		    MODE="$1"
		    shift
		    ;;
		--component)
		    shift
		    COMPONENT="$1"
		    shift
		    ;;
		--version)
		    shift
		    VERSION="$1"
		    shift
		    ;;
		--configs)
		    shift
		    CONFIGS="$@"
		    shift
		    ;;
		-? | --usage | --help )
		    usage
		    exit 0
		    ;;

		-* )
		    echo "Unrecognized option: $1" >&2
		    usage
		    exit $E_BAD_OPTION
		    ;;

		* )
		    break
		    ;;
	    esac
	done

	log "using ${MODE} and ${COMPONENT}"

	[ -z "${MODE}" ] && error "no mode parameter" && exit 1;

	if [ -z "${CONFIGS}" ]; then
		CONFIGS=$(ls -1 ${CONF_SCRIPT_DIR}/config-*)
	else
		local configs=""
		for c in ${CONFIGS}; do
			configs="${configs} ${CONF_SCRIPT_DIR}/config-${c}"
			echo $c
		done
		CONFIGS=${configs}
	fi

	log "configs: ${CONFIGS}"

	for c in ${CONFIGS}; do
		rm -f ./p
		if [ "${MODE}" == "migrate" ]; then
			./${c}
			ptxdist-${VERSION} ${MODE};
			[ "${?}" -ne "0" ] && error "${MODE} for ${c} failed, give up ..." && exit 1;
		else
			./${c}
			[ "${?}" -ne "0" ] && error "using ${c} failed, give up ..." && exit 1
			./p ${MODE} ${COMPONENT};
			[ "${?}" -ne "0" ] && error "${MODE} ${COMPONENT} for ${c} failed, give up ..." && exit 1;
		fi
	done;

	exit 0;
}

main $@

