#!/bin/bash
# SPDX-License-Identifier: GPL-2.0-or-later
#
# File:        prepare-release.sh
#
# Copyright (C) 2016-2022 TQ-Systems GmbH <oss@ew.tq-group.com>, D-82229 Seefeld, Germany.
# Author: Markus Niebel
#
# Description: A utility script to install a PTXdist version
#
###############################################################################

# DEBUGGING
# set -e
set -C # noclobber

VERSION=
VERBOSE=0

# RETURN VALUES/EXIT STATUS CODES
readonly E_BAD_OPTION=254
readonly E_UNKNOWN=255

function usage () {
    echo "Usage is as follows:"
    echo
    echo "$PROGRAM <--usage|--help|-?>"
    echo "    Prints this usage output and exits."
    echo "$PROGRAM --version <Release Tag> [--last <Last Release Tag>]"
    echo "    create release tag and put log beginning from <Last Release Tag>"
    echo "    in tag comment"
    echo
}

function log () {
    if [ $VERBOSE -eq 1 ]; then
        echo $1
    fi
}

function error () {
    echo $1 >&2
}

function main () {
	VERSION=
	LAST=

	# Process command-line arguments.
	while test $# -gt 0; do
	    case $1 in

		--version )
		    shift
		    VERSION=$1
		    shift
		    ;;

		--last )
		    shift
		    LAST=$1
		    shift
		    ;;

		--verbose )
		    shift
		    VERBOSE=1
		    ;;

		-? | --usage | --help )
		    usage
		    exit
		    ;;

		-* )
		    error "Unrecognized option: $1" 
		    usage
		    exit $E_BAD_OPTION
		    ;;

		* )
		    break
		    ;;
	    esac
	done

	if [ -z ${VERSION} ]
	then
		error "missing version"
		usage
		exit $E_BAD_OPTION
	fi

	if [ -z ${LAST} ]
	then
		REV=$(echo ${VERSION##*.})
		LAST=${VERSION%.*}.$(printf "%#04d" $(expr ${REV} - 1))
	fi

	git show-ref --quiet --tags ${LAST} 2>/dev/null
	if [ "${?}" -ne "0" ]
	then
		error "${LAST} is not a tag"
		usage
		exit $E_BAD_OPTION
	fi


	git tag -a ${VERSION} -m "${VERSION}" -m "$(git log --date=short --format="%ad %an: %s" ${LAST}..HEAD)"
}

main $@


