#!/bin/bash
# SPDX-License-Identifier: GPL-2.0-or-later
#
# File:        tools_lib_ptxdist.sh
#
# Copyright (C) 2017-2022 TQ-Systems GmbH <oss@ew.tq-group.com>, D-82229 Seefeld, Germany.
# Author: Markus Niebel
#
# Description: helper fuctions for the BSP.
#
###############################################################################

check_ptxdist() {
	echo "Checking for ptxdist ..."
	ptxdist --version >/dev/null 2>&1
	if [ "$?" -ne "0" ]; then
		echo "'ptxdist' not found! Please install." >&2
		exit 255;
	fi
	return 0
}

check_ptxdist_version() {
	local VERSION=$1

	if [ -z ${VERSION} ]; then
		echo "usage 'check_ptxdist_version' VERSION." >&2
		exit 255;
	fi

	local PTXDIST=ptxdist-${VERSION}
	${PTXDIST} --version >/dev/null 2>&1
	[ $? -ne 0 ] && echo "'${PTXDIST}' not found! Please install." && exit ${E_UNKNOWN};

	echo "using ${PTXDIST}"

	rm -f ./p
	ln -s /usr/local/bin/${PTXDIST} ./p

	return 0
}

ptxdist_has_working_print() {
	local VERSION=$1
	local PRINT_SUPPORTED="no"

	if [ -z ${VERSION} ]; then
		echo "usage 'ptxdist_has_working_print' VERSION." >&2
		exit 255;
	fi

	# starting with 2016.12 ptxdist print does not work
	# starting with 2017.03 ptxdist print works again
	local VER_YEAR=$(echo ${VERSION} | awk -F"." '{print $1}')
	local VER_MONTH=$(echo ${VERSION} | awk -F"." '{print $2}')

	[ "${VER_YEAR}" -lt "2016" ] && PRINT_SUPPORTED="yes"
	[ "${VER_YEAR}" -eq "2016" ] && [ "${VER_MONTH}" -lt "12" ] && PRINT_SUPPORTED="yes"
	[ "${VER_YEAR}" -eq "2017" ] && [ "${VER_MONTH}" -gt "2" ] && PRINT_SUPPORTED="yes"
	[ "${VER_YEAR}" -gt "2017" ] && PRINT_SUPPORTED="yes"

	[ "${PRINT_SUPPORTED}" != "yes" ] && return -1

	return 0
}

ptxdist_has_license_report() {
	local VERSION=$1
	local REPORT_SUPPORTED="no"

	if [ -z ${VERSION} ]; then
		echo "usage 'ptxdist_has_license-report' VERSION." >&2
		exit 255;
	fi

	# starting with 2014.02.0 ptxdist has make license-report
	local VER_YEAR=$(echo ${VERSION} | awk -F"." '{print $1}')
	local VER_MONTH=$(echo ${VERSION} | awk -F"." '{print $2}')

	[ "${VER_YEAR}" -gt "2014" ] && REPORT_SUPPORTED="yes"
	[ "${VER_YEAR}" -eq "2014" ] && [ "${VER_MONTH}" -ge "2" ] && REPORT_SUPPORTED="yes"

	[ "${REPORT_SUPPORTED}" != "yes" ] && return -1

	return 0
}

ptxdist_has_doc_generation() {
	local VERSION=$1
	local DOC_SUPPORTED="no"

	if [ -z ${VERSION} ]; then
		echo "usage 'ptxdist_has_doc_generation' VERSION." >&2
		return 255;
	fi

	# starting with 2016.07.0 ptxdist has doc-latex / docs-html
	local VER_YEAR=$(echo ${VERSION} | awk -F"." '{print $1}')
	local VER_MONTH=$(echo ${VERSION} | awk -F"." '{print $2}')

	[ "${VER_YEAR}" -gt "2016" ] && DOC_SUPPORTED="yes"
	[ "${VER_YEAR}" -eq "2016" ] && [ "${VER_MONTH}" -ge "7" ] && DOC_SUPPORTED="yes"

	[ "${DOC_SUPPORTED}" != "yes" ] && return -1

	return 0
}

check_config_files() {
	if [ "$#" -lt "2" ] || [ "$#" -gt "3" ]; then
		echo "usage: 'check_config_files' PLATFORMCONFIG PTXCONFIG [COLLECTION]." >&2
		exit 255;
	fi

	local PLATFORMCONFIG=$1
	local PTXCONFIG=$2
	local COLLECTION=""

	! [ -z $3 ] && COLLECTION=$3
	
	if [ -f $PLATFORMCONFIG ]; then
		echo "configuring for platform-tq-$PLATFORM: $(grep "PTXCONF_PLATFORM=" $PLATFORMCONFIG)"
	else
		echo "trying to use non existing $PLATFORMCONFIG" >&2
		exit 255
	fi

	if [ -f $PTXCONFIG ]; then
		echo "configuring for $(grep "PTXCONF_PROJECT="  $PTXCONFIG)"
	else
		echo "trying to configure for non exisiting $PTXCONFIG"  >&2
		exit 255
	fi

	if [ ! -z "$COLLECTION" ]; then
		if [ -f $COLLECTION ]; then
			echo "use collection $COLLECTION"
		else
			echo "trying to configure with non existing $COLLECTION"  >&2
			exit 255
		fi
	fi

	return 0
}

## apply_config ${PLATFORMCONFIG} ${PTXCONFIG} ${COLLECTION}
apply_config () {
	if [ "$#" -lt "2" ] || [ "$#" -gt "3" ]; then
		echo "usage 'apply_config' PLATFORMCONFIG PTXCONFIG [COLLECTION]." >&2
		exit 255;
	fi

	local PLATFORMCONFIG=$1
	local PTXCONFIG=$2
	local COLLECTION=""

	! [ -z $3 ] && COLLECTION=$3

	#
	# clean old config links
	#
	rm -f selected_collectionconfig
	rm -f selected_platformconfig
	rm -f selected_ptxconfig

	ptxdist --force platform ${PLATFORMCONFIG}
	ptxdist --force select ${PTXCONFIG}
	if [ ! -z "${COLLECTION}" ]; then
		ptxdist --force collection ${COLLECTION}
	fi

	return 0
}

get_ptxconfig_var() {
	VAR=${1}
	local VALUE=$(ptxdist print ${VAR} 2>/dev/null | tr -d '"')

	[ -z "${VALUE}" ] && \
		VALUE=$(ptxdist --force print ${VAR} 2>/dev/null | tr -d '"')

	if [ -z "${VALUE}" ]
	then
		VALUE=$(grep -w ${VAR} selected_ptxconfig)
		VALUE=$(echo ${VALUE##*=\"})
		VALUE=$(echo ${VALUE%%\"})
	fi

	if [ -z "${VALUE}" ]; then
		echo "no config selected! Please correct." >&2
		exit 255;
	fi

	echo ${VALUE}
	return 0
}

get_platform_var() {
	VAR=${1}
	local VALUE=$(ptxdist print ${VAR} 2>/dev/null | tr -d '"')

	[ -z "${VALUE}" ] && \
		VALUE=$(ptxdist --force print ${VAR} 2>/dev/null | tr -d '"')

	if [ -z "${VALUE}" ]
	then
		VALUE=$(grep -w ${VAR} selected_platformconfig)
		VALUE=$(echo ${VALUE##*=\"})
		VALUE=$(echo ${VALUE%%\"})
	fi

	if [ -z "${VALUE}" ]; then
		echo "no config selected! Please correct." >&2
		exit 255;
	fi

	echo "${VALUE}"
	return 0
}

get_ptxconfig_version() {
	local CONFIG_FILE_VERSION=$(ptxdist print PTXCONF_CONFIGFILE_VERSION | tr -d '\\"')
	[ -z "${CONFIG_FILE_VERSION}" ] && \
		CONFIG_FILE_VERSION=$(ptxdist --force print PTXCONF_CONFIGFILE_VERSION 2>/dev/null | tr -d '\\"')

	if [ -z "${CONFIG_FILE_VERSION}" ]
	then
		CONFIG_FILE_VERSION=$(grep -w PTXCONF_CONFIGFILE_VERSION selected_ptxconfig)
		CONFIG_FILE_VERSION=$(echo ${CONFIG_FILE_VERSION##*=\"})
		CONFIG_FILE_VERSION=$(echo ${CONFIG_FILE_VERSION%%\"})
	fi

	if [ -z "${CONFIG_FILE_VERSION}" ]; then
		echo "no config selected! Please correct." >&2
		exit 255;
	fi

	echo ${CONFIG_FILE_VERSION}
	return 0
}

get_platformconfig_version() {
	local PLATFORM_FILE_VERSION=$(ptxdist print PTXCONF_PLATFORMCONFIG_VERSION | tr -d '\\"')
	[ -z "${PLATFORM_FILE_VERSION}" ] && \
		PLATFORM_FILE_VERSION=$(ptxdist --force print PTXCONF_PLATFORMCONFIG_VERSION 2>/dev/null | tr -d '\\"')

	if [ -z "${PLATFORM_FILE_VERSION}" ]
	then
		PLATFORM_FILE_VERSION=$(grep -w PTXCONF_PLATFORMCONFIG_VERSION selected_platformconfig)
		PLATFORM_FILE_VERSION=$(echo ${PLATFORM_FILE_VERSION##*=\"})
		PLATFORM_FILE_VERSION=$(echo ${PLATFORM_FILE_VERSION%%\"})
	fi

	if [ -z "${PLATFORM_FILE_VERSION}" ]; then
		echo "no platform selected! Please correct." >&2
		exit 255;
	fi

	echo "${PLATFORM_FILE_VERSION}"
	return 0
}

get_platform_name() {
	local PLATFORM_NAME=$(ptxdist print PTXCONF_PLATFORM | tr -d '\\"')
	[ -z "${PLATFORM_NAME}" ] && \
		PLATFORM_NAME=$(ptxdist --force print PTXCONF_PLATFORM 2>/dev/null | tr -d '\\"')

	if [ -z "${PLATFORM_NAME}" ]
	then
		PLATFORM_NAME=$(grep -w PTXCONF_PLATFORM selected_platformconfig)
		PLATFORM_NAME=$(echo ${PLATFORM_NAME##*=\"})
		PLATFORM_NAME=$(echo ${PLATFORM_NAME%%\"})
	fi

	if [ -z "${PLATFORM_NAME}" ]; then
		echo "no platform name configured! Please correct.";
		exit 255;
	fi

	echo "${PLATFORM_NAME}"
	return 0
}

